/*
    Copyright 2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include "emea.h"

#include <QtGui/QApplication>
#include <QtCore/QDebug>

#include <dopplerproducer.h>
#include <gagereader.h>
#include <evaluationpath.h>
#include <resulttospectrum.h>

class EMEADoppler : public EMEAWindow
{
	public:
		EMEADoppler( QWidget* p ) : EMEAWindow( p ) {
			qDebug() << "EMEADoppler::EMEADoppler(" << p << ")";
		}

		void setupChain( int threads ) {
			qDebug() << "EMEADoppler::setupChain(" << threads << ")";
			GageReader* reader;
			addToChainStart( reader = objectInNewThread<GageReader>() );
			reader->setDispatching( NSCore::DataSetProducer::DispatchEqual );
			DopplerProducer* prod;
			addToChainStart( prod = objectInNewThread<DopplerProducer>() );
			prod->setDispatching( NSCore::DataSetProducer::DispatchEqual );
			QList<NSCore::DataSetChainMember*> evals;
			for ( int i=0; i<threads; ++i )
				evals << objectInNewThread<NSEvaluation::EvaluationPath>();
			addToChain( evals );
			ResultsToSpectrum* res = objectInNewThread<ResultsToSpectrum>();
			connect( res, SIGNAL( newSpectrum( AbstractSpectrumModel* ) ), spectrumGuiCreator(), SLOT( newSpectrum( AbstractSpectrumModel* ) ) );
			addToChain( res );
		}

		void autoStart() {
			//reader->readAllDataSets();
			qDebug() << endl << " I can't autostart the readout because there are two readers and I don't know which one to start..." << endl;
		}
};

int main( int argc, char** argv )
{

	QApplication app( argc, argv );

	EMEAWindow* w = new EMEADoppler( 0 );
	w->show();

	int ret = app.exec();

	delete w;

	return ret;
};

