/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.

*/

#ifndef DATASETCHAIN_TEST_H
#define DATASETCHAIN_TEST_H

#include "datasetchain.h"

namespace Test {

	class Producer : public NSCore::DataSetProducer
	{
		Q_OBJECT
		public:
			Producer() : NSCore::DataSetProducer(), toProduce( 0 ) {}

			int toProduce;

			QDomNode saveToXML( QDomDocument doc ) { return doc.createElement( "bla" ); }
			bool restoreFromXML( QDomNode node ) { return node.nodeName() == "bla"; }

			QString debug() const { return "Producer"; }

		public slots:
			void produceData() {
				NSCore::AcqisitionDataSet data( 2, 10 );
				for ( int i=0; i<data.channels(); ++i ) {
					double* ch = data.getRawChannel( i );
					for ( int j=0; j<data.samples(); ++j )
						ch[ j ] = rand()*2.0/RAND_MAX-1;
				}
				qDebug() << "Producer::produceData() produced" << &data;
				enqueue( data );
				toProduce--;
				if ( toProduce > 0 )
					QTimer::singleShot( 0, this, SLOT( produceData() ) );
				else
					QTimer::singleShot( 1000, QCoreApplication::instance(), SLOT( quit() ) );
			}
	};

	class Consumer : public NSCore::DataSetChainMember
	{
		bool dataArrived( NSCore::AcqisitionDataSet& data ) {
			qDebug() << "Consumer::dataArrived(" << &data << ")";
			for ( int i=0; i<data.channels(); ++i ) {
				double val=0;
				double* ch = data.getRawChannel( i );
				for ( int j=0; j<data.samples(); ++j )
					val += ch[ j ];
				val /= data.samples();
				qDebug() << " Channel" << i << ": Average =" << val;
				data.addResult( QString( "channel %1/average" ).arg( i ), val );
				data.addResult( QString( "channel %1/samples processed" ).arg( i ), data.samples() );
			}
			data.debugPrint();
			//qDebug() << " done.";
			return true;
		}
		public:
			QDomNode saveToXML( QDomDocument doc ) { return doc.createElement( "bla" ); }
			bool restoreFromXML( QDomNode node ) { return node.nodeName() == "bla"; }
			QString debug() const { return "Consumer"; }

	};

	class ChainMember : public NSCore::DataSetChainMember
	{
		public:
			bool dataArrived( NSCore::AcqisitionDataSet& data ) {
				qDebug() << "ChainMember::dataArrived(" << &data << ")";
				int oldgeneration = data.currentGenerationNumber();
				data.newGeneration( "averaged", data.channels(), data.samples() );
				for ( int i=0; i<data.channels(); ++i ) {
					double* newch = data.getRawChannel( i );
					double* oldch = data.getRawChannel( i, oldgeneration );

					double avg[ 3 ];
					avg[ 0 ] = 0.0;
					avg[ 1 ] = 0.0;
					avg[ 2 ] = oldch[ 0 ];

					for ( int j=0; j<data.samples(); ++j ) {
						avg[ 0 ] = avg[ 1 ];
						avg[ 1 ] = avg[ 2 ];
						avg[ 2 ] = oldch[ j+1 ];

						double tmp = ( avg[ 0 ] + avg[ 1 ] + avg[ 2 ] ) / 3.0;
						newch[ j ] = tmp;
					}

					double val=0;
					for ( int j=0; j<data.samples(); ++j )
						val += newch[ j ];
					val /= data.samples();
					qDebug() << " Channel" << i << ": Average =" << val;
					data.addResult( QString( "channel %1/average" ).arg( i ), val );

				}
				data.popPrefix( "averaged" );
				return true;
			}
			QString debug() const { return "ChainMember"; }
			QDomNode saveToXML( QDomDocument ) { return QDomNode(); }
			bool restoreFromXML( QDomNode node ) { return node.nodeName() == "bla"; }
	};

};

#endif // DATASETCHAIN_TEST_H
