/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/

#include "resultviewer.h"
#include "resultviewer.moc"
#include "resultviewer_p.h"
#include "resultviewer_p.moc"

#include <QtCore/QDebug>
#include <QtGui/QVBoxLayout>

class ResultViewer::ResultViewerPrivate : public NSCore::DataSetChainMember {
public:
	ResultViewerPrivate( QWidget* w, QObject* parent )
		: NSCore::DataSetChainMember( parent )
		, pw( w )
		, layout( new QVBoxLayout( pw ) )
	{
		setObjectName( "Result Viewer" );
		w->setObjectName( "Result Viewer" );
		//qDebug() << "ResultViewerPrivate::ResultViewerPrivate() pw =" << pw << " pw->layout =" << pw->layout();
		_slowdown.start();
	}

	QString debug() const { return "ResultViewer"; }
	QDomNode saveToXML( QDomDocument ) { return QDomNode(); }
	bool restoreFromXML( QDomNode ) { return false; }

	bool dataArrived( NSCore::AcqisitionDataSet& data ) {
		if ( _slowdown.elapsed() > 200 ) {
			recursiveResultWorker( data.resulttree() );
			_slowdown.restart();
		}
		return true;
	}

	void recursiveResultWorker( NSCore::NamedQVariantTreeItem* item ) {
		QString path = item->path();
		if ( item->type() != QVariant::Invalid ) {
			if ( !widgets.contains( path ) ) {
				ResultWidget* tmp = new ResultWidget( path, pw );
				layout->addWidget( tmp );
				widgets.insert( path, tmp );
				tmp->show();
				pw->update();
			}
			widgets[ path ]->addValue( item->toDouble() );
		}
		foreach( QString name, item->childlist() )
			recursiveResultWorker( item->child<NSCore::NamedQVariantTreeItem*>( name ) );
	}

	QHash<QString,ResultWidget*> widgets;

	QWidget* pw;
	QVBoxLayout* layout;
	QTime _slowdown;
};

ResultViewer::ResultViewer( QWidget* p )
	: QWidget( p )
	, d( new ResultViewerPrivate( this, this ) )
{
}
ResultViewer::~ResultViewer() {
}

NSCore::DataSetChainMember* ResultViewer::chainMember() {
	return d;
}

