/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef RESULTVIEWER_P_H
#define RESULTVIEWER_P_H

#include <QtCore/QDebug>

#include <QtGui/QWidget>
#include <QtGui/QHBoxLayout>
#include <QtGui/QLabel>
#include <QtGui/QDoubleSpinBox>

class ResultWidget : public QWidget
{
	Q_OBJECT
	public:
		ResultWidget( QString name, QWidget* p ) : QWidget( p ), min( 1000 ), max( -1000 ), avg( 0 ) {
			//qDebug() << "ResultWidget::ResultWidget(" << name << "," << p << ")";
			QHBoxLayout* layout = new QHBoxLayout( this );

			layout->addWidget( new QLabel( name, this ) );

			QDoubleSpinBox* tmp = new QDoubleSpinBox( 0 );
			connect( this, SIGNAL( minChanged( double ) ), tmp, SLOT( setValue( double ) ) );
			layout->addWidget( tmp );
			tmp->setRange( -1000, 1000 );
			tmp->setEnabled( false );
			tmp = new QDoubleSpinBox( 0 );
			connect( this, SIGNAL( lastChanged( double ) ), tmp, SLOT( setValue( double ) ) );
			layout->addWidget( tmp );
			tmp->setRange( -1000, 1000 );
			tmp->setEnabled( false );
			tmp = new QDoubleSpinBox( 0 );
			connect( this, SIGNAL( avgChanged( double ) ), tmp, SLOT( setValue( double ) ) );
			layout->addWidget( tmp );
			tmp->setRange( -1000, 1000 );
			tmp->setEnabled( false );
			tmp = new QDoubleSpinBox( 0 );
			connect( this, SIGNAL( maxChanged( double ) ), tmp, SLOT( setValue( double ) ) );
			layout->addWidget( tmp );
			tmp->setRange( -1000, 1000 );
			tmp->setEnabled( false );
		}
		~ResultWidget() {
		}

		double min, max, avg;
		void addValue( double n ) {
			//qDebug() << "ResultWidget::addValue(" << n << ")";
			if (  n < min ) {
				min = n;
				emit minChanged( min );
			}
			if (  n > max ) {
				max = n;
				emit maxChanged( max );
			}
			avg = (  avg + n ) / 2.0;
			emit avgChanged( avg );
			emit lastChanged( n );
		}

	signals:
		void minChanged( double );
		void maxChanged( double );
		void lastChanged( double );
		void avgChanged( double );
};

#endif // RESULTVIEWER_P_H
