/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/
#ifndef FFTW_H
#define FFTW_H

#include "plugins_interface.h"

#include <fftw3.h>

class DFT_R2HC : public NSEvaluation::Modifier
{
	public:
		DFT_R2HC( NSCore::QTreeItem* );
		~DFT_R2HC();

		void prepare( NSCore::AcqisitionDataSet& );

		void modify( NSCore::AcqisitionDataSet&, int );

		QString type() const { return "DFT_Real2HalfComplex"; }

	private:
		int _samples;
		fftw_plan _p;
		double *_in, *_out;
};

class DFT_NiceAnalysis : public NSEvaluation::Modifier
{
	public:
		DFT_NiceAnalysis( NSCore::QTreeItem* );
		~DFT_NiceAnalysis();

		void prepare( NSCore::AcqisitionDataSet& );

		void modify( NSCore::AcqisitionDataSet&, int );

		QString type() const { return "DFT_NiceAnalysis"; };

	private:
		int _samples;
		fftw_plan _p;
		double *_in, *_out;
};

class DFT_HC2R : public NSEvaluation::Modifier
{
	public:
		DFT_HC2R( NSCore::QTreeItem* );
		~DFT_HC2R();

		void prepare( NSCore::AcqisitionDataSet& );

		void modify( NSCore::AcqisitionDataSet&, int );

		QString type() const { return "DFT_HalfComplex2Real"; }

	private:
		int _samples;
		fftw_plan _p;
		double *_in, *_out;
};

class HC_LowPass : public NSEvaluation::Modifier
{
	public:
		HC_LowPass( NSCore::QTreeItem* );

		void modify( NSCore::AcqisitionDataSet&, int );

		QString type() const { return "HC_LowPass"; }
};

#endif // FFTW_H
