/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ANALYZERS_H
#define ANALYZERS_H

#include "plugins_interface.h"

#include "interpolation_helpers.h"
#include <mathexpression.h>

class InterpolatedMinimumAnalyzerLagrange : public NSEvaluation::Analyzer
{
	public:
		InterpolatedMinimumAnalyzerLagrange( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "InterpolatedMinimumAnalyzerLagrange"; }

	private:
		double L( double x, int k, int x0 ) const;
		double p( double x, int x0, double* ) const;
		double min_interpolated3( int x_min, double* ) const;
};

class InterpolatedMinimumAnalyzer : public NSEvaluation::Analyzer, private PolynomInterpolate, private Minimum
{
	public:
		InterpolatedMinimumAnalyzer( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "InterpolatedMinimumAnalyzer"; }

	private:
		double function( double n ) {
			return PolynomInterpolate::polint( n );
		}
};

class InterpolatedMaximumAnalyzer : public NSEvaluation::Analyzer, private PolynomInterpolate, private Maximum
{
	public:
		InterpolatedMaximumAnalyzer( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "InterpolatedMaximumAnalyzer"; }

	private:
		double function( double n ) {
			return PolynomInterpolate::polint( n );
		}
};

class ZeroLineAnalyzer : public NSEvaluation::Analyzer {
	public:
		ZeroLineAnalyzer( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "ZeroLineAnalyzer"; }
};

class ThresholdTriggerAnalyzer : public NSEvaluation::Analyzer, private PolynomInterpolate
{
	public:
		ThresholdTriggerAnalyzer( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "ThresholdTriggernalyzer"; }
};

class CFlinAnalyzer : public NSEvaluation::Analyzer
{
	public:
		CFlinAnalyzer( NSCore::QTreeItem* );
		virtual ~CFlinAnalyzer();

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "CFlinAnalyzer"; }

	private:
		NSEvaluation::Analyzer* _minimumanalyzer;
};

class CFAnalyzer : public NSEvaluation::Analyzer, private PolynomInterpolate
{
	public:
		CFAnalyzer( NSCore::QTreeItem* );
		~CFAnalyzer();

		QString type() const { return "CFAnalyzer"; }

		void analyze( NSCore::AcqisitionDataSet& data );

	private:
		coord points[ 3 ];
};

/*class CFSplineAnalyzer : public NSEvaluation::Analyzer, private Spline
{
	public:
		CFSplineAnalyzer( NSCore::QTreeItem* );
		~CFSplineAnalyzer();

		QString type() const { return "CFSplineAnalyzer"; }

		void analyze( NSCore::AcqisitionDataSet& data );

	private:
		double y( int );
		NSEvaluation::Analyzer* _minimumanalyzer;
};*/

class MeanAnalyzer : public NSEvaluation::Analyzer
{
	public:
		MeanAnalyzer( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "MeanAnalyzer"; }
};

class StatisticsAnalyzer : public NSEvaluation::Analyzer
{
	public:
		StatisticsAnalyzer( NSCore::QTreeItem* );
		void analyze( NSCore::AcqisitionDataSet& data );
		QString type() const { return "StatisticsAnalyzer"; }
};

class IntegralAnalyzer : public NSEvaluation::Analyzer
{
	public:
		IntegralAnalyzer( NSCore::QTreeItem* );
		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "IntegralAnalyzer"; }

	private:
		NSCore::MathTreeRoot* _begin;
		NSCore::MathTreeRoot* _end;
};

class NewResultAnalyzer : public NSEvaluation::Analyzer
{
	public:
		NewResultAnalyzer( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& );

		QString type() const { return "NewResultAnalyzer"; }

	private:
		NSCore::MathTreeRoot* _math;
};

#endif // ANALYZERS_H

