/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.

*/
#include <QtCore/QDebug>
#include <math.h>
#include <iostream>

#include "modifiers.h"

int main(  int argc, char** argv )
{
	qDebug() << "Averager Test starting...";

	const int size = 500;

	//int* bla;
	srand( 5 /* *bla */ );

	double* data = new double[ size ];
	for ( int i=0; i<size; ++i ) {
		data[ i ] = rand() * 1.0 / RAND_MAX;
	}
	data[ size/2 ] = 2;

	double* out1 = new double[ size ];
	simpleAverager( 5, size, data, out1 );

	double* out2 = new double[ size ];
	simpleAveragerV2( 5, size, data, out2 );

	double* out3 = new double[ size ];
	simpleAveragerV2( 10, size, data, out3 );

	int n = 5;
	int nsize = 2 * n + 1;
	double* weights = new double[ nsize ];
	double s = 0.75;
	double sum = 0;
	for ( int i=0; i<nsize; i++ ) {
		weights[ i ] = exp( - pow( i-n,2 ) / ( 2 * pow( s,2 ) ) );
		std::cerr << i << "\t" << weights[ i ] << std::endl;
		sum += weights[ i ];
	}
	for ( int i=0; i<nsize; ++i ) {
		weights[ i ] /= sum;
		std::cerr << i << "\t" << weights[ i ] << std::endl;
	}
	std::cerr << " Sum = " << sum << std::endl;
	/*weights[ 0 ] = 0.1;
	weights[ 1 ] = 0.2;
	weights[ 2 ] = 0.4;
	weights[ 3 ] = 0.2;
	weights[ 4 ] = 0.1;*/
	double* out4 = new double[ size ];
	weightedAverager( n, size, data, out4, weights );

	for ( int i=0; i<size; ++i )
		std::cout << i << "\t" << data[ i ]
			<< "\t" << out1[ i ]
			<< "\t" << out2[ i ]
			<< "\t" << out3[ i ]
			<< "\t" << out4[ i ]
			<< std::endl;
	std::cout << std::endl << std::endl;

/*	MySpline* int_low = new MySpline( data, size, 0, 0 );
	for ( int i=1; i<size-2; ++i ) {
		for ( double j=0; j<1; j+=0.1 )
			std::cout << i+j
				<< "\t" << int_low->splint( i+j )
				<< "\t" << func( i+j, size)
				<< "\t" << fabs( ( int_low->splint( i+j ) - func( i+j, size ) ) )
				<< std::endl;
	}*/


	qDebug() << "...finished.";
	return 0;
}

