/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef RESULTTOSPECTRUM_P_H
#define RESULTTOSPECTRUM_P_H

#include <mathexpression.h>
#include <QtCore/QThread>

class ResultsToSpectrum;
class AbstractSpectrumModel;

class QTimer;

/**
 * @brief A path of expressions together with a spectrummodel
 */
class ResultAction : public NSCore::MathTreeRoot
{
	Q_OBJECT
	public:
		ResultAction( ResultsToSpectrum* parent =0 );
		~ResultAction();

	public slots:
		void data( NSCore::AcqisitionDataSet );

	public:

		QString guiString() const { return QString( "Spectrum: %1" ).arg( objectName() ); }

		QStringList guiActions() const { return NSCore::MathTreeRoot::guiActions() << "Remove Me"; }
		void guiExecAction( QString, NSCore::GuiServer_Interface* );

		AbstractSpectrumModel* _model;

	private:
		QDomNode saveToXML( QDomDocument );
		bool restoreFromXML( QDomNode );
		QString debug() const { return QString( "ResultAction: %1" ).arg( objectName() ); }
};


class ResultThread: public QObject
{
	Q_OBJECT
	public:
		ResultThread() : _parent( 0 ) {
			//qDebug() << "ResultThread::ResultThread()";
			_timer = new QTimer( this );
			_timer->setInterval( 100 );
			connect( _timer, SIGNAL( timeout() ), this, SLOT( parseXml() ) );
			//_timer->setSingleShot( true );
			_timer->start();
		}
		~ResultThread() {
			//qDebug() << "ResultThread::~ResultThread()";
		}

		void addXml( QDomNode node ) {
			_tocreate.append( node );
			//_timer->start();
		}

		void setParent( ResultsToSpectrum* p ) { _parent = p; }

	signals:
		void newSpectrum( AbstractSpectrumModel* );

	private slots:
		void parseXml() {
			if ( _tocreate.size() == 0 ) return;
			//qDebug() << "ResultThread::parseXml()";
			while ( _tocreate.size() > 0 ) {
				QDomNode node = _tocreate.takeFirst();
				ResultAction* tmp = new ResultAction();
				connect( _parent, SIGNAL( data( NSCore::AcqisitionDataSet ) ), tmp, SLOT( data( NSCore::AcqisitionDataSet ) ) );
				_parent->addChild( tmp );
				tmp->fromXml( node );
				emit newSpectrum( tmp->_model );
			}
		}

	private:
		QList<QDomNode> _tocreate;
		QTimer* _timer;
		ResultsToSpectrum* _parent;
};

#endif // RESULTTOSPECTRUM_P_H

